package uk.co.sportquest.jsfbeans.helper;

import java.util.List;
import javax.faces.context.FacesContext;
import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;
import org.ajax4jsf.model.Range;
import org.ajax4jsf.model.SequenceRange;

/*
 * @author Brendan Healey
 * 
 * Works with a RichFaces rich:extendedDataTable, like this:
 * 
 * private RichLazyDataModel<Event> model;
 * 
 *     model = new RichLazyDataModel<Event>() {
 *           @Override
 *           public List<Event> getDataList(int firstRow, int numRows) {
 *               return selectedEntity.getEventList(firstRow, numRows);
 *           }
 *           @Override
 *           public Object getKey(Event t) {
 *               return t.getId();
 *           }
 *           @Override
 *           public int getTotalCount() {
 *               return selectedEntity.getEventListCount().intValue();
 *           }
 *       };
 */
public abstract class RichLazyDataModel<T> extends ExtendedDataModel<T> {

    protected SequenceRange cachedRange = null;
    protected Integer cachedRowCount = null;
    protected List<T> cachedList = null;
    protected Object rowKey;

    public abstract List<T> getDataList(int firstRow, int numRows);

    public abstract Object getKey(T t);

    public abstract int getTotalCount();

    @Override
    public void walk(FacesContext ctx, DataVisitor dv, Range range, Object argument) {

        SequenceRange sr = (SequenceRange) range;

        if (cachedList == null || !equalRanges(cachedRange, sr)) {

            int firstRow = sr.getFirstRow();
            int numRows = sr.getRows();

            //SQLog.log("firstRow: " + firstRow + " numRows: " + numRows);
            
            cachedList = getDataList(firstRow, numRows);
            cachedRange = sr;
        }

        for (T t : cachedList) {
            dv.process(ctx, getKey(t), argument);
        }
    }

    @Override
    public void setRowKey(Object rowKey) {
        this.rowKey = rowKey;
    }

    @Override
    public Object getRowKey() {
        return rowKey;
    }

    @Override
    public boolean isRowAvailable() {
        return (getRowData() != null);
    }

    @Override
    public int getRowCount() {
        if (cachedRowCount == null) {
            cachedRowCount = getTotalCount();
        }
        return cachedRowCount;
    }

    @Override
    public T getRowData() {
        for (T t : cachedList) {
            if (getKey(t).equals(this.getRowKey())) {
                return t;
            }
        }
        return null;
    }

    protected static boolean equalRanges(SequenceRange range1, SequenceRange range2) {
        if (range1 == null || range2 == null) {
            return range1 == null && range2 == null;
        } else {
            return range1.getFirstRow() == range2.getFirstRow()
                    && range1.getRows() == range2.getRows();
        }
    }

    @Override
    public int getRowIndex() {
        return cachedList.indexOf(rowKey);
    }

    @Override
    public void setRowIndex(int rowIndex) {
        setRowKey(cachedList.get(rowIndex));
    }

    @Override
    public Object getWrappedData() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setWrappedData(Object data) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
